﻿$engine JScript
$nodebug
$uname settings
$dname Настройки Снегопата
$addin global
$addin stdlib

global.connectGlobals(SelfScript)

// Базовый класс для сохраняемого параметра
Param = stdlib.Class.extend({
    construct: function(profileName, formName, rebootOnChange)
    {
        this.profileName = profileName
        this.formName = formName
        this.rebootOnChange = rebootOnChange
    },
    valueOnForm:    function(form)      { return form[this.formName] },
    valueToForm:    function(form, v)   { form[this.formName] = v },
    valueInProfile: function(profile)   { return profile.getValue(this.profileName) },
    valueToProfile: function(profile, v){ profile.setValue(this.profileName, v) },
    validate:       function(val)       { return true },
    isEqual:        function(v1, v2)    { return v1 == v2 }
});

// Класс наследник для параметра - набора битового флагов, каждый из которых отображается в флажок на форме
ParamFlags = Param.extend({
    construct: function(profileName, rebootOnChange /* , ... тут перечисляются имена флажков на форме */)
    {
        this._super(profileName, '', rebootOnChange)
        this.flagsNames = []
        for(var k = 2; k < arguments.length; k++)
            this.flagsNames.push(arguments[k])
    },
    valueOnForm: function(form)
    {
        var v = 0, flag = 1;
        for(var k in this.flagsNames)
        {
            v |= form[this.flagsNames[k]] ? flag : 0
            flag <<= 1
        }
        return v
    },
    valueToForm: function(form, v)
    {
        var flag = 1;
        for(var k in this.flagsNames)
        {
            form[this.flagsNames[k]] = 0 != (v & flag)
            flag <<= 1
        }
    }
});

ParamColor = Param.extend({
    validate: function(val)
    {
        if(ЗначениеВСтрокуВнутр(val.Вид) != '{"#",7c626d2b-6cc1-4f6c-9367-352a9da94a2e,0}')
        {
            MessageBox("Параметр " + this.formName + ". Для цвета можно задавать только абсолютные значения!")
            return false
        }
        return true
    },
    isEqual: function(v1, v2)
    {
        return ЗначениеВСтрокуВнутр(v1) == ЗначениеВСтрокуВнутр(v2)
    }
});

Params = [
    new Param('EnableCustomGrouping', 'ПользовательскиеГруппировки', true),
    new Param('QueryColors', 'РаскрашиватьСтроки', true),
    new Param('Autohint', 'Телепатировать', true),
    new Param('Autoreplace', 'Автозамены', true),
    new Param('HookStdList', 'ПерехватШтатногоСписка', true),
    new Param('ActionOnSelectScript', 'ВариантДействияПриВыбореСкрипта', true),
    new Param('EditScriptCommand', 'КомандаЗапускаРедактора', true),
    new Param('ListWidth', 'ШиринаСписка', false),
    new Param('QuickActivateCharsCount', 'СимволовДляАктивации', false),
    new Param('ParamsAutoShow', 'ParamsAutoShow', false),
    new Param('AllowFilterInSmartList', 'AllowFilterInSmartList', false),
    new ParamFlags('UseLangs', true, 'фАнглийский', 'фРусский'),
    new ParamColor('MultiLineBackground', 'ЦветФонаМногострочныхСтрок', true)
]

Form = stdlib.Class.extend({
    construct: function()
    {
        this.rebootOnChange = false
        this.profile = profileRoot.getFolder('Snegopat/Settings')
        this.form = loadScriptForm("core\\std\\forms\\settings.ssf", this)
        this.form.ОткрытьМодально()
    },
    ПриОткрытии: function()
    {
        for(var k in Params)
            Params[k].valueToForm(this.form, Params[k].valueInProfile(this.profile))
        this.ПользовательскиеГруппировкиПриИзменении()
        this.ВариантДействияПриВыбореСкриптаПриИзменении()
    },
    ПользовательскиеГруппировкиПриИзменении: function()
    {
        var form = this.form
        if(!form.ПользовательскиеГруппировки)
            form.РаскрашиватьСтроки = false
        form.ЭлементыФормы.РаскрашиватьСтроки.Доступность = form.ПользовательскиеГруппировки
        form.ЭлементыФормы.ЦветФонаМногострочныхСтрок.Доступность = form.РаскрашиватьСтроки
        
        if(!form.Телепатировать)
        {
            form.Автозамены = false
            form.ПерехватШтатногоСписка = false
        }
        var ctrls = ['Автозамены', 'ПерехватШтатногоСписка', 'ШиринаСписка', 'СимволовДляАктивации']
        for(var k in ctrls)
            form.ЭлементыФормы[ctrls[k]].Доступность = form.Телепатировать
    },
    ВариантДействияПриВыбореСкриптаПриИзменении: function() 
    {
        var enableCmd = (this.form.ВариантДействияПриВыбореСкрипта == 1);
        this.form.ЭлементыФормы.НадписьКомандаЗапускаРедактора.Доступность = enableCmd
        this.form.ЭлементыФормы.КомандаЗапускаРедактора.Доступность = enableCmd
    },
    фРусскийПриИзменении: function()
    {
        this.form.фАнглийский = true
    },
    фАнглийскийПриИзменении: function()
    {
        this.form.фРусский = true
    },
    ОтменаНажатие: function()
    {
        this.form.Закрыть()
    },
    ОкНажатие: function()
    {
        // Сначала получим все значения и проверим их
        var values = {}
        for(var k in Params)
        {
            var currentValue = Params[k].valueOnForm(this.form)
            if(!Params[k].validate(currentValue))
                return
            values[k] = currentValue
        }
        // Все параметры прошли проверку, будем их сохранять
        for(var k in Params)
        {
            // Если при смене параметра нужна перезагрузка, проверим, не изменился ли он
            if(Params[k].rebootOnChange && !Params[k].isEqual(values[k], Params[k].valueInProfile(this.profile)))
                this.rebootOnChange = true
            Params[k].valueToProfile(this.profile, values[k])
        }
        this.form.Закрыть()
    }
});

function macrosОткрытьНастройкиСнегопата()
{
    var f = new Form()
    if(f.rebootOnChange)
        MessageBox("Необходимо перезапустить Конфигуратор для вступления изменений в силу.", mbIconWarning, "Снегопат")
}
